/*
 * XMLLoader.cpp
 *
 *  Created on: 12.3.2013
 *      Author: petr
 */

#include "XMLLoader.h"

namespace FiniteAutomat {

const pugi::char_t* XMLLoader::ELEMENT_ASSEMBLER = "assembler";
const pugi::char_t* XMLLoader::ELEMENT_FILTER = "filter";
const pugi::char_t* XMLLoader::ELEMENT_LOADER = "loader";
const pugi::char_t* XMLLoader::ELEMENT_MACHINE = "machine";
const pugi::char_t* XMLLoader::ELEMENT_PARAM = "param";
const pugi::char_t* XMLLoader::ELEMENT_PROCESSOR = "processor";
const pugi::char_t* XMLLoader::ELEMENT_ROOT = "fa";
const pugi::char_t* XMLLoader::ELEMENT_STATE = "state";
const pugi::char_t* XMLLoader::ELEMENT_STATEAGREGATED = "stateagregated";
const pugi::char_t* XMLLoader::ELEMENT_TEST = "test";
const pugi::char_t* XMLLoader::ELEMENT_TRANSITION = "transition";

const pugi::char_t* XMLLoader::ATTR_AGREGATEDNAME = "agrmachine";
const pugi::char_t* XMLLoader::ATTR_CNAME = "class";
const pugi::char_t* XMLLoader::ATTR_FINAL = "isfinal";
const pugi::char_t* XMLLoader::ATTR_FROM = "from";
const pugi::char_t* XMLLoader::ATTR_NAME = "name";
const pugi::char_t* XMLLoader::ATTR_PRIORITY = "priority";
const pugi::char_t* XMLLoader::ATTR_STARTER = "starter";
const pugi::char_t* XMLLoader::ATTR_TO = "to";
const pugi::char_t* XMLLoader::ATTR_VALUE = "value";

XMLLoader::XMLLoader() {
	fLoad = NULL;
	fTest = NULL;
	fFil = NULL;
	fAss = NULL;
	fProc = NULL;

}

Machine* XMLLoader::loadMachine(pugi::xml_node node, Machine* machine) {
	NIM states;

	// vytvoreni instanci stavu
	loadSates(node, machine, states);
	loadStatesAgregated(node, machine, states);

	// sestaveni prechodu
	pugi::xml_node transitionNode = node.child(ELEMENT_TRANSITION);

	while (transitionNode) {
		loadTransition(transitionNode, machine, states);

		transitionNode = transitionNode.next_sibling(ELEMENT_TRANSITION);
	}

	// nastaveni vychoziho stavu
	machine->setInitialState((State*) states[node.attribute(ATTR_STARTER).as_string()]);

	return machine;
}

Machine* XMLLoader::loadSates(pugi::xml_node node, Machine* machine, NIM& nim) {
	pugi::xml_node stateNode = node.child(ELEMENT_STATE);

	while (stateNode) {
		State* state = new State(machine);
		setStdStateValues(stateNode, state, nim);

		// nastaveni procesoru
		pugi::xml_node processorNode = stateNode.child(ELEMENT_PROCESSOR);
		if (processorNode) {
			// vytvoreni instance procesoru a jeji nastaveni
			Processor* processor = fProc->factory(processorNode.attribute(ATTR_CNAME).as_string());
			setParameters(processorNode, *processor);

			state->setProcessor(processor);
			delete processor;
		}

		stateNode = stateNode.next_sibling(ELEMENT_STATE);
	}

	return machine;
}

Machine* XMLLoader::loadStatesAgregated(pugi::xml_node node,
		Machine* machine, NIM& nim) {
	pugi::xml_node stateNode = node.child(ELEMENT_STATEAGREGATED);

	while (stateNode) {
		std::string machineName = stateNode.attribute(ATTR_AGREGATEDNAME).as_string();
		StateAgregated* state = new StateAgregated(machine, (Machine*) machines[machineName]);
		setStdStateValues(stateNode, state, nim);

		// nastaveni procesoru
		pugi::xml_node assemblerNode = stateNode.child(ELEMENT_ASSEMBLER);
		if (assemblerNode) {
			// vytvoreni instance procesoru a jeji nastaveni
			Assembler* assembler = fAss->factory(assemblerNode.attribute(ATTR_CNAME).as_string());
			setParameters(assemblerNode, *assembler);

			state->setAssembler(assembler);
			delete assembler;
		}

		stateNode = stateNode.next_sibling(ELEMENT_STATE);
	}

	return machine;
}

Transition* XMLLoader::loadTransition(pugi::xml_node node, Machine* machine,
		NIM& nim) {

	std::string fromName = node.attribute(ATTR_FROM).as_string(), toName = node.attribute(ATTR_TO).as_string();
	State *from = (State*) nim[fromName], *to = (State*) nim[toName];
	Transition* trans = new Transition(from, to, node.attribute(ATTR_PRIORITY).as_int());

	// nastaveni loaderu
	pugi::xml_node loaderNode = node.child(ELEMENT_LOADER);
	Loader* loader = fLoad->factory(loaderNode.attribute(ATTR_CNAME).as_string());

	setParameters(loaderNode, *loader);
	trans->setLoader(loader);
	delete loader;

	// nastaveni dalsich hodnot
	loadFilters(node, trans);
	loadTests(node, trans);

	return trans;
}

Transition* XMLLoader::loadFilters(pugi::xml_node node,
		Transition* transition) {

	// prochazeni a registrace filtru
	pugi::xml_node filterNode = node.child(ELEMENT_FILTER);

	while (filterNode) {
		// vygenerovani a nastaveni filtru
		Filter* filter = fFil->factory(filterNode.attribute(ATTR_CNAME).as_string());
		setParameters(filterNode, *filter);

		transition->getFilters().push_back(filter);

		filterNode = filterNode.next_sibling(ELEMENT_FILTER);
	}

	return transition;
}

Transition* XMLLoader::loadTests(pugi::xml_node node, Transition* transition) {
	// prochazeni a registrace filtru
	pugi::xml_node testNode = node.child(ELEMENT_TEST);

	while (testNode) {
		// vygenerovani a nastaveni filtru
		Test* test = fTest->factory(testNode.attribute(ATTR_CNAME).as_string());
		setParameters(testNode, *test);

		transition->getTests().push_back(test);

		testNode = testNode.next_sibling(ELEMENT_TEST);
	}

	return transition;
}

void XMLLoader::setParameters(pugi::xml_node node, ParamLoadable& item) {
	pugi::xml_node param = node.child(ELEMENT_PARAM);

	while (param) {
		std::string name = param.attribute(ATTR_NAME).as_string(), value = param.attribute(ATTR_VALUE).as_string();
		item.setParameter(name, value);

		param = param.next_sibling(ELEMENT_PARAM);
	}
}

XMLLoader::~XMLLoader() {
	// TODO Auto-generated destructor stub
}

void XMLLoader::setStdStateValues(pugi::xml_node stateNode, State* state, NIM& nim) {
	// vytvoreni stavu a zapis do seznamu
	std::string name(stateNode.attribute(ATTR_NAME).as_string());

	nim[name] = state;

	// vyhodnoceni finalniho stavu
	state->setFinal(stateNode.attribute(ATTR_FINAL).as_bool());
}

Machine* XMLLoader::load(std::string fileName) {
	// priprava dat
	pugi::xml_document document;
	document.load_file(fileName.c_str());

	// reset hodnot
	machines.clear();

	/**
	 * nejprve musime vytvorit prazdne automaty
	 * pokud bychom to neudelali, hrozilo by vyzadani neexistujiciho automatu pro zanoreni
	 */
	pugi::xml_node root = document.child(ELEMENT_ROOT);
	pugi::xml_node machineNode = root.child(ELEMENT_MACHINE);

	Machine* currMachine;

	while (machineNode) {
		// nacteni jmena a vytvoreni automatu
		std::string name(machineNode.attribute(ATTR_NAME).as_string());
		currMachine = new Machine();

		// zapis automatu do indexu jmen
		machines[name] = currMachine;

		machineNode = machineNode.next_sibling(ELEMENT_MACHINE);
	}

	/**
	 * prazdne automaty jsou vytvoreny
	 * projdeme seznam jeste jednou a nacteme definice
	 */
	machineNode = root.child(ELEMENT_MACHINE);

	while (machineNode) {
		// nacteme jmeno automatu a z nej zjistime ukazatel na vytvorenou instanci
		std::string name(machineNode.attribute(ATTR_NAME).as_string());
		currMachine = (Machine*) machines[name];

		// nacteni automatu
		loadMachine(machineNode, currMachine);

		// pokracujeme na dalsi automat
		machineNode = machineNode.next_sibling(ELEMENT_MACHINE);
	}

	std::string mainName(root.attribute(ATTR_STARTER).as_string());
	return (Machine*) machines[mainName];
}

} /* namespace FiniteAutomat */
