/*
 * Table.cpp
 *
 *  Created on: 3.2.2013
 *      Author: petr
 */

#include "Table.h"

namespace SQLRepresenter {

Table::Table(Database* database, string name) :
		SQLBase(database) {
	this->name = name;
	nodeType = NODE_TABLE;
}

Table::~Table() {
	// odstraneni sloupcu, indexu, cizich klicu a triggeru, ktere tabulka tvori
	NodeList childs = childNodes(NODE_COLUMN | NODE_INDEX | NODE_TRIGGER | NODE_FOREIGN_KEY);
	NodeList::iterator pos = childs.begin(), end = childs.end();

	while (pos != end) {
		delete *pos;

		pos++;
	}

	// odstraneni cizich klicu, kde je tabulka referencni
	NodeList fks = parentNodes(NODE_FOREIGN_KEY);
	pos = fks.begin();
	end = fks.end();

	while (pos != end) {
		delete *pos;

		pos++;
	}
}

Column* Table::createColumn(string name) {
	// nemohou byt dva sloupce stejneho jmena
	if (hasColumn(name))
		return NULL;

	Column* retVal = new Column((Database*) this->graph(), name);

	retVal->nodeType = NODE_COLUMN;

	// musi se vytvorit zavislost mezi tabulkou a sloupcem
	addChild(retVal, MATE_MAJOR_PARENT);

	return retVal;
}

ForeignKey* Table::createForeignKey(string name, ColumnList columns, Table* refTable, ColumnList refColumns) {
	ForeignKey* retVal = new ForeignKey((Database*) graph(), name);

	// musime vytvorit vazby na lokalni tabulku a sloupce
	addChild(retVal, MATE_MAJOR_PARENT);

	ColumnList::iterator pos = columns.begin(), end = columns.end();

	while (pos != end) {
		retVal->addChild(*pos, MATE_FOREIGN_TARGET);
		pos++;
	}

	// musime vytvorit vazby na referenci tabulku a sloupce
	if (refTable != NULL)
		retVal->addChild(refTable, MATE_FOREIGN_REF);

	pos = refColumns.begin();
	end = refColumns.end();

	while (pos != end) {
		retVal->addChild(*pos, MATE_FOREIGN_REF);
		pos++;
	}

	return retVal;
}

Index* Table::createIndex(string name, ColumnList columns) {
	// dva indexy se nemohou jmenovat stejne - musi se zkontrolovat jestli neni duplicitni
	if (hasIndex(name))
		return NULL;

	Index *retVal = new Index((Database*) this->graph(), name);

	addChild(retVal, MATE_MAJOR_PARENT);

	// index nesmi byt prazdny - pridaji se sloupce
	ColumnList::iterator pos = columns.begin(), end = columns.end();

	while (pos != end) {
		retVal->addChild(*pos, MATE_INDEXED);

		pos++;
	}

	return retVal;
}

void Table::dropColumn(string name) {
	// nalezeni zaznamu
	NodeList columns = childNodes(NODE_COLUMN);

	dropItem(columns, name);
}

void Table::dropForeignKey(string name) {
	NodeList foreigns = childNodes(NODE_FOREIGN_KEY);

	dropItem(foreigns, name);
}

void Table::dropIndex(string name) {
	// nalezeni zaznamu
	NodeList indexes = childNodes(NODE_INDEX);

	dropItem(indexes, name);
}

ColumnList Table::getColumns() {
	ColumnList retVal;
	NodeList nodes = childNodes(NODE_COLUMN);
	NodeList::iterator pos = nodes.begin(), end = nodes.end();

	while (pos != end) {
		retVal.push_back((Column*) *pos);
		pos++;
	}

	return retVal;
}

IndexList Table::getIndexes() {
	IndexList retVal;
	NodeList nodes = childNodes(NODE_INDEX);
	NodeList::iterator pos = nodes.begin(), end = nodes.end();

	while (pos != end) {
		retVal.push_back((Index*) *pos);
		pos++;
	}

	return retVal;
}

Index* Table::getPrimary() {
	IndexList indexes = getIndexes();
	IndexList::iterator pos = indexes.begin(), end = indexes.end();

	Index *retVal = NULL;

	while (pos != end && retVal == NULL) {
		if ((*pos)->indexType == INDEX_TYPE_PRIMARY)
			retVal = *pos;

		pos++;
	}

	return retVal;
}

bool Table::hasColumn(string name) {
	return (findItem(childNodes(NODE_COLUMN), name) == NULL) ? false : true;
}

Column* Table::getColumn(string name) {
	return (Column*) findItem(childNodes(NODE_COLUMN), name);
}

Check* Table::createCheck(string name) {
	Check* retVal;
	retVal = new Check((Database*) graph(), name);

	addChild(retVal, MATE_MAJOR_PARENT);

	return retVal;
}

Trigger* Table::createTrigger(string name) {
	Trigger* retVal;

	retVal = new Trigger((Database*) graph());
	retVal->name = name;

	this->addChild(retVal, MATE_MAJOR_PARENT);

	return retVal;
}

void Table::dropTrigger(string name) {
	dropItem(childNodes(NODE_TRIGGER), name);
}

TriggerList Table::getTriggers() {
	TriggerList retVal;
	NodeList triggers = childNodes(NODE_TRIGGER);
	NodeList::iterator pos = triggers.begin(), end = triggers.end();

	while (pos != end) {
		retVal.push_back((Trigger*) *pos);

		pos++;
	}

	return retVal;
}

bool Table::hasIndex(string name) {
	return (findItem(childNodes(NODE_INDEX), name) == NULL) ? false : true;
}

Index* Table::getIndex(string name) {
	return (Index*) findItem(childNodes(NODE_INDEX), name);
}

Trigger* Table::getTrigger(string name) {
	return (Trigger*) findItem(childNodes(NODE_TRIGGER), name);
}

void Table::dropCheck(string name) {
	// nalezeni zaznamu
	NodeList indexes = childNodes(NODE_CHECK);

	dropItem(indexes, name);
}

ForeignKey* Table::getForeignKey(string name) {
	return (ForeignKey*) findItem(childNodes(NODE_FOREIGN_KEY), name);
}

ForeignList Table::getForeignKeys() {
	ForeignList retVal;
	NodeList foreigns = childNodes(NODE_FOREIGN_KEY);
	NodeList::iterator pos = foreigns.begin(), end = foreigns.end();

	while (pos != end) {
		retVal.push_back((ForeignKey*) *pos);

		pos++;
	}

	return retVal;
}

Check* Table::getCheck(string name) {
	return (Check*) findItem(childNodes(NODE_CHECK), name);
}

CheckList Table::getChecks() {
	CheckList retVal;
	NodeList checks = childNodes(NODE_CHECK);
	NodeList::iterator pos = checks.begin(), end = checks.end();

	while (pos != end) {
		retVal.push_back((Check*) *pos);

		pos++;
	}

	return retVal;
}

bool Table::hasForeignKey(string name) {
	return (findItem(childNodes(NODE_FOREIGN_KEY), name) == NULL) ? false : true;
}

bool Table::hasCheck(string name) {
	return (findItem(childNodes(NODE_CHECK), name) == NULL) ? false : true;
}

bool Table::hasTrigger(string name) {
	return (findItem(childNodes(NODE_TRIGGER), name) == NULL) ? false : true;
}

Table& Table::operator =(const Table& o) {
	storageType = o.storageType;

	*((SQLBase*) this) = o;

	return *this;
}

bool Table::operator ==(const Table& o) {
	return storageType == o.storageType && *((SQLBase*) this) == (const SQLBase&) o;
}

}

