/*
 * DiffPackage.h
 *
 *  Created on: 17.4.2013
 *      Author: petr
 */

#ifndef DIFFPACKAGE_H_
#define DIFFPACKAGE_H_

#include <algorithm>
#include <string>
#include <vector>
#include <ctime>
#include <zip.h>
#include <sha.h>
#include <hex.h>
#include <files.h>
#include <iostream>
#include <sstream>
#include <map>
#include <memory>
#include <boost/filesystem.hpp>
#include <exception>

#include "Representation/Database.h"
#include "Representation/XMLAdapter.h"
#include "Representation/LocalAdapter.h"
#include "Representation/ConflictFinder.h"
#include "Configurator.h"
#include "pugixml.hpp"

using namespace std;
using namespace pugi;
using namespace boost::filesystem3;

class DiffPackage {

public:

	typedef vector<DiffPackage*> Packages;
	typedef vector<string> PackageUuids;
	typedef map<string, DiffPackage*> DiffIndex;
	typedef map<string, DiffPackage*> BranchList;

	struct META_INFO {
			string name;				// jmeno tvurce
			string email;				// email na tvurce
			string branch;				// identifikace vetve
			tm time;					// cas vytvoreni
			PackageUuids parents;		// seznam UUID predku
		};

private:

	DiffPackage();
	virtual ~DiffPackage();

	/**
	 * pool se vsemi nactenymi balicky
	 */
	static DiffIndex pool;

	/**
	 * seznam vetvi, tkere byly nalezeny v ulozisti
	 * balicek je vzdy nejnovejsi nalezeny (HEAD vetve)
	 */
	static BranchList branches;

	/**
	 * reprezentace delta databaze
	 */
	SQLRepresenter::Database* diffDb;

	/**
	 * metainformace
	 */
	META_INFO metaInfo;

	/**
	 * vygeneruje XML dokument s meta informacemi
	 */
	string generateMetaXML();

	/**
	 * balicek je pouze pro cteni, pokud je nacitan z disku
	 */
	bool readOnly;

	/**
	 * seznam balicku, ktere jsou rodici
	 */
	Packages parents;

	/**
	 * seznam balicku, ktere jsou potomci
	 */
	Packages children;

	/**	 */
	string name;

	/**
	 * nacte ZIP archiv
	 */
	static bool loadFile(path source);

	/**
	 * vygeneruje jmeno vetve
	 */
	static string generateBranchName();

	/**
	 * zkontroluje, jestli id vetve existuje
	 * pokud ne, tak ho zapise do seznamu
	 */
	static void writeBranchId(DiffPackage *package);

	/**
	 * provede vlastni rekurznivni sestaveni
	 */
	void doAssemble(SQLRepresenter::Database *db, SQLRepresenter::Conflicts conflicts);

	/**
	 * najde prvni balicek z druhe vetve a vraci ho
	 * balicek v parametru je nejnovejsi nalezeny balicek
	 */
	DiffPackage::Packages* findFirstPackage(string branch, DiffPackage *currentPackage);

	void getPath(Packages &path, DiffPackage *limit);

	/**
	 * zkontroluje, jestli je balicek v historii
	 */
	bool inHistory(DiffPackage* package);

	static void generateDeltaChain(SQLRepresenter::DeltaChain &chain, Packages packages);

public:

	static const char* NODE_ROOT;
	static const char* NODE_USER;
	static const char* NODE_EMAIL;
	static const char* NODE_PARENTS;
	static const char* NODE_PARENT;
	static const char* NODE_TIME;
	static const char* NODE_YEAR;
	static const char* NODE_MONTH;
	static const char* NODE_DAY;
	static const char* NODE_HOUR;
	static const char* NODE_MINUTE;
	static const char* NODE_SECOND;
	static const char* NODE_BRANCH;
	static const char* NODE_CONFLICTS;
	static const char* NODE_CONFLICT;

	static const char* EXTENSION;

	static const char* ATTR_DB;
	static const char* ATTR_ID;
	static const char* ATTR_METHOD;

	static const char* METHOD_CURRENT;
	static const char* METHOD_OTHER;

	static const char* META_FILE;
	static const char* DATA_FILE;

	/**
	 * zaznamenane konflikty
	 */
	SQLRepresenter::Conflicts conflicts;

	/**
	 * vytvori balicek
	 *
	 * pokud je parent == NULL, pak je vytvorena nova vetev
	 */
	static DiffPackage *createPackage(DiffPackage *parent);

	/**
	 * vraci balicek dle jmena
	 */
	static DiffPackage *getPackage(string name);

	/**
	 * vypocita SHA1 souboru
	 */
	static string calculateSHA1(string filename);

	/**
	 * nacte pool balicku
	 */
	static void loadPool(Configuration config);

	/**
	 * prida dalsiho rodice k balicku
	 */
	void addParent(DiffPackage *parent);

	/**
	 * sestavi databazi z z predchozich balicku
	 */
	SQLRepresenter::Database *assembleDatabase();

	/**
	 * vraci delta databazi balicku
	 */
	SQLRepresenter::Database *getDatabase();

	void getDeltaChain(SQLRepresenter::DeltaChain &current, SQLRepresenter::DeltaChain &other, string branch);

	/**
	 * vraci seznam balicku (vetvi), ktere je mozne sloucit
	 */
	Packages getMergeables();

	/**
	 * vraci kopii metainfo
	 */
	META_INFO getMetaInfo();

	/**
	 * vraci referenci na meta info
	 */
	META_INFO &meta();

	/**
	 * raci jmeno balicku
	 */
	string getName();

	/**
	 * nastavi aktualni cas
	 */
	void setCurrentTime();

	/**
	 * nastavi novou databazi
	 */
	void setDatabase(SQLRepresenter::Database *database);

	/**
	 * zapise balicek do souboru
	 */
	string writeFile(string target);

	/**
	 * operator prirazeni
	 */
	DiffPackage &operator =(const DiffPackage& o);
};

#endif /* DIFFPACKAGE_H_ */
